@@ -2,15 +2,19 @@
 
 #written by Jeremy M. Beaulieu
 
-#Fits the Ornstein-Uhlenbeck model of continuous characters evolving under discrete selective 
-#regimes. The input is a tree of class "phylo" that has the regimes as internal node labels 
-#and a trait file. The trait file must be in the following order: Species names, Regime, and 
+#Fits the Ornstein-Uhlenbeck model of continuous characters evolving under discrete selective
+#regimes. The input is a tree of class "phylo" that has the regimes as internal node labels
+#and a trait file. The trait file must be in the following order: Species names, Regime, and
 #continuous trait. Different models can be specified -- Brownian motion (BM), multiple rate BM (BMS)
-#global OU (OU1), multiple regime OU (OUM), multiple sigmas (OUMV), multiple alphas (OUMA), 
-#and the multiple alphas and sigmas (OUMVA). 
+#global OU (OU1), multiple regime OU (OUM), multiple sigmas (OUMV), multiple alphas (OUMA),
+#and the multiple alphas and sigmas (OUMVA).
 
 OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA", "TrendyM", "TrendyMS"), simmap.tree=FALSE, root.age=NULL, scaleHeight=FALSE, root.station=TRUE, clade=NULL, mserr="none", starting.vals=NULL, diagn=FALSE, quiet=FALSE, warn=TRUE){
-	
+
+    if(model=="BMS" & root.station==TRUE){
+        warning("By setting root.station=TRUE, you have specified the group means model of Thomas et al. 2006", call.=FALSE, immediate.=TRUE)
+    }
+
     if(is.factor(data[,3])==TRUE){
         stop("Check the format of the data column. It's reading as a factor.", .call=FALSE)
     }
@@ -20,7 +24,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
             stop("Looks like your tree is producing negative branching times. Must input known root age of tree.", .call=FALSE)
         }
     }
-    
+
     if(!is.null(starting.vals[1])){
         if(model == "OU1" | model == "OUM" | model == "OUMV" | model == "OUMA" | model == "OUMVA"){
             if(length(starting.vals)<2){
@@ -28,7 +32,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
             }
         }
     }
-    
+
     #Makes sure the data is in the same order as the tip labels
 	if(mserr=="none" | mserr=="est"){
 		data<-data.frame(data[,2], data[,3], row.names=data[,1])
@@ -47,7 +51,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
             }
 		}
 	}
-    
+
 	#Values to be used throughout
 	n=max(phy$edge[,1])
 	ntips=length(phy$tip.label)
@@ -76,42 +80,42 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 		tot.states<-factor(colnames(phy$mapped.edge))
 		tip.states<-factor(data[,1])
 		data[,1]<-as.numeric(tip.states)
-		
+
 		#A boolean for whether the root theta should be estimated -- default is that it should be.
 		root.station=root.station
 		#Obtains the state at the root
 		root.state=which(colnames(phy$mapped.edge)==names(phy$maps[[1]][1]))
 		##Begins the construction of the edges matrix -- similar to the ouch format##
 		edges=cbind(c(1:(n-1)),phy$edge, MakeAgeTable(phy, root.age=root.age))
-		
+
 		if(scaleHeight==TRUE){
 			edges[,4:5]<-edges[,4:5]/max(MakeAgeTable(phy, root.age=root.age))
 		}
 		edges=edges[sort.list(edges[,3]),]
-		
+
 		#Resort the edge matrix so that it looks like the original matrix order
 		edges=edges[sort.list(edges[,1]),]
 	}
 	if(simmap.tree==FALSE){
-		#Obtain a a list of all the regime states. This is a solution for instances when tip states and 
+		#Obtain a a list of all the regime states. This is a solution for instances when tip states and
 		#the internal nodes are not of equal length:
-		tot.states<-factor(c(phy$node.label,as.character(data[,1])))
-		k<-length(levels(tot.states))
-		int.states<-factor(phy$node.label)
-		phy$node.label=as.numeric(int.states)		
-		tip.states<-factor(data[,1])
-		data[,1]<-as.numeric(tip.states)
-		
+		tot.states <- factor(c(phy$node.label,as.character(data[,1])))
+		k <- length(levels(tot.states))
+		int.states <- factor(phy$node.label)
+		phy$node.label=as.numeric(int.states)
+		tip.states <- factor(data[,1])
+		data[,1] <- as.numeric(tip.states)
+
 		#A boolean for whether the root theta should be estimated -- default is that it should be.
 		root.station=root.station
-		if (is.character(model)) {			
+		if (is.character(model)) {
 			if (model == "BM1"| model == "OU1"){
 				##Begins the construction of the edges matrix -- similar to the ouch format##
 				#Makes a vector of absolute times in proportion of the total length of the tree
 				phy$node.label<-sample(c(1:k), phy$Nnode, replace=T)
 				int.states=length(levels(tip.states))
 				#Since we only really have one global regime, make up the internal nodes -- this could be improved
-				phy$node.label<-as.numeric(int.states)				
+				phy$node.label<-as.numeric(int.states)
 				#Obtain root state -- for both models assume the root state to be 1 since no other state is used even if provided in the tree
 				root.state<-1
 				#New tree matrix to be used for subsetting regimes
@@ -120,7 +124,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 					edges[,4:5]<-edges[,4:5]/max(MakeAgeTable(phy, root.age=root.age))
 				}
 				edges=edges[sort.list(edges[,3]),]
-				
+
 				regime <- matrix(0,nrow=length(edges[,1]),ncol=k)
 				regime[,1]<-1
 				if (k>1) {
@@ -128,7 +132,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 				}
 				edges=cbind(edges,regime)
 			}
-			else{			
+			else{
 				#Obtain root state and internal node labels
 				root.state<-phy$node.label[1]
 				int.state<-phy$node.label[-1]
@@ -138,12 +142,12 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 					edges[,4:5]<-edges[,4:5]/max(MakeAgeTable(phy, root.age=root.age))
 				}
 				edges=edges[sort.list(edges[,3]),]
-				
+
 				mm<-c(data[,1],int.state)
-				regime <- matrix(0,nrow=length(mm),ncol=length(unique(mm)))
+				regime <- matrix(0,nrow=length(mm),ncol=k)
 				#Generates an indicator matrix from the regime vector
 				for (i in 1:length(mm)) {
-					regime[i,mm[i]] <- 1 
+					regime[i,mm[i]] <- 1
 				}
 				#Finishes the edges matrix
 				edges=cbind(edges,regime)
@@ -152,12 +156,12 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 		#Resort the edge matrix so that it looks like the original matrix order
 		edges=edges[sort.list(edges[,1]),]
 	}
-	
-	#Initializes an integer specifying whether or not the trendy model is to be invoked. Assume 0, unless otherwise stated later:
+
+    #Initializes an integer specifying whether or not the trendy model is to be invoked. Assume 0, unless otherwise stated later:
 	trendy=0
 	#Data:
 	x<-as.matrix(data[,2])
-	
+
 	#Matches the model with the appropriate parameter matrix structure
 	if (is.character(model)) {
 		index.mat<-matrix(0,2,k)
@@ -169,7 +173,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 				index.mat[1,1:k]<-np+2
 			}
 			else{
-				index.mat[1,1:k]<-np+1				
+				index.mat[1,1:k]<-np+1
 			}
 			index.mat[2,1:k]<-1
 			param.count<-np+1
@@ -267,7 +271,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 				index.mat[1,1:k]<-np+2
 			}
 			else{
-				index.mat[1,1:k]<-np+1				
+				index.mat[1,1:k]<-np+1
 			}
 			index.mat[2,1:k]<-1
 			index.mat[3,1:k]<-2:(np-1)
@@ -299,16 +303,16 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 			warning("You might not have enough data to fit this model well", call.=FALSE, immediate.=TRUE)
 		}
 	}
-	
+
 	#Likelihood function for estimating model parameters
-	dev<-function(p, index.mat, edges, mserr, trendy){
-				
+	dev <- function(p, index.mat, edges, mserr, trendy){
+
 		p <- exp(p)
 		Rate.mat[] <- c(p, 1e-10)[index.mat]
 		N<-length(x[,1])
 		root.par.index=length(p)
 		V<-varcov.ou(phy, edges, Rate.mat, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight)
-		if (any(is.nan(diag(V))) || any(is.infinite(diag(V)))) return(1000000)		
+		if (any(is.nan(diag(V))) || any(is.infinite(diag(V)))) return(1000000)
 		if(mserr=="known"){
 			diag(V)<-diag(V)+(data[,3]^2)
 		}
@@ -327,9 +331,9 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 			#However, sometimes this fails (not sure yet why) so I just toggle between this and another approach:
 			if(!is.finite(DET)){
 				DET <- determinant(V, logarithm=TRUE)
-				logl <- -.5*(t(W%*%theta-x)%*%pseudoinverse(V)%*%(W%*%theta-x))-.5*as.numeric(DET$modulus)-.5*(N*log(2*pi))
+				logl <- -.5*(t(x-W%*%theta)%*%pseudoinverse(V)%*%(x-W%*%theta))-.5*as.numeric(DET$modulus)-.5*(N*log(2*pi))
 			}else{
-				logl <- -.5*(t(W%*%theta-x)%*%pseudoinverse(V)%*%(W%*%theta-x))-.5*as.numeric(DET)-.5*(N*log(2*pi))
+				logl <- -.5*(t(x-W%*%theta)%*%pseudoinverse(V)%*%(x-W%*%theta))-.5*as.numeric(DET)-.5*(N*log(2*pi))
 			}
 		}else{
 			E_a <- Inf
@@ -341,18 +345,18 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 			#However, sometimes this fails (not sure yet why) so I just toggle between this and another approach:
 			if(!is.finite(DET)){
 				DET <- determinant(V, logarithm=TRUE)
-				logl <- -.5*(t(E_a-x)%*%pseudoinverse(V)%*%(E_a-x))-.5*as.numeric(DET$modulus)-.5*(N*log(2*pi))
+				logl <- -.5*(t(x-E_a)%*%pseudoinverse(V)%*%(x-E_a))-.5*as.numeric(DET$modulus)-.5*(N*log(2*pi))
 			}else{
-				logl <- -.5*(t(E_a-x)%*%pseudoinverse(V)%*%(E_a-x))-.5*as.numeric(DET)-.5*(N*log(2*pi))
+				logl <- -.5*(t(x-E_a)%*%pseudoinverse(V)%*%(x-E_a))-.5*as.numeric(DET)-.5*(N*log(2*pi))
 			}
-		}		
+		}
 		#When the model includes alpha, the values of V can get too small, the modulus does not seem correct and the loglik becomes unstable. This is one solution:
 		if(!is.finite(logl)){
 			return(10000000)
 		}
 		return(-logl)
 	}
-	
+
 	if(quiet==FALSE){
 		cat("Initializing...", "\n")
 	}
@@ -361,9 +365,9 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 	ub = 20
 	lower = rep(-20, np)
 	upper = rep(20, np)
-	
+
 	opts <- list("algorithm"="NLOPT_LN_SBPLX", "maxeval"="1000000", "ftol_rel"=.Machine$double.eps^0.5)
-	
+
 	if(model == "OU1" | model == "OUM" | model == "OUMV" | model == "OUMA" | model == "OUMVA"){
 		if(scaleHeight==TRUE){
 			d <- max(diag(vcv.phylo(phy)))
@@ -378,7 +382,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 		init.index.mat<-matrix(0,2,k)
 		init.index.mat[1,1:k]<-1
 		init.index.mat[2,1:k]<-2
-		if(model == "OU1" | model == "OUM"){
+		if(model == "OU1"){
 			edges.tmp=edges
 			if(dim(edges.tmp)[2] < 6){
 				edges.tmp <- cbind(edges.tmp, rep(1, length(edges.tmp[,1])))
@@ -390,6 +394,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 		}else{
 			edges.tmp=edges
 		}
+
 		#Initial value for alpha is just the half life based on the entire length of the tree:
         if(is.null(starting.vals)){
             init <- nloptr(x0=log(c(log(2)/max(branching.times(phy)),sig)), eval_f=dev, lb=init.lower, ub=init.upper, opts=opts, index.mat=init.index.mat, edges=edges.tmp, mserr=mserr, trendy=trendy)
@@ -410,7 +415,7 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 			ip<-c(ip,sig)
 			lower = c(lower,0)
 			upper = c(upper,ub)
-		}		
+		}
 		if(quiet==FALSE){
 			cat("Finished. Begin thorough search...", "\n")
 		}
@@ -422,10 +427,10 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 			phy$edge.length<-(phy$edge.length/d)
 		}
 		#Starting values follow from phytools:
-		C.mat<-vcv.phylo(phy)
-		a<-as.numeric(colSums(solve(C.mat))%*%x/sum(solve(C.mat)))
+		C.mat <- vcv.phylo(phy)
+		a <- as.numeric(colSums(solve(C.mat))%*%x/sum(solve(C.mat)))
         if(is.null(starting.vals)){
-            sig<-as.numeric(t(x-a)%*%solve(C.mat)%*%(x-a)/n)
+            sig <- as.numeric(t(x-a)%*%solve(C.mat)%*%(x-a)/n)
         }else{
             sig <- starting.vals
         }
@@ -483,25 +488,33 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 		tmp$res<-W%*%theta-x
 		#Joins the vector of thetas with the vector of standard errors into a 2 column matrix for easy extraction at the summary stage
 		tmp$theta.est<-cbind(theta,se)
-		#Returns final GLS solution
-		tmp
+
+        #Calculates R-sq according to Judge et al. 1985 eq. 2.3.16:
+        residuals <- tmp$res
+        fitted.vals <- W%*%theta
+        Y <- residuals + fitted.vals
+        one <- matrix(1, length(Y), 1)
+        a <- as.numeric(pseudoinverse(t(one) %*% pseudoinverse(V) %*% one) %*% (t(one) %*% pseudoinverse(V) %*% Y))
+        Rsq <- 1-(t(residuals) %*% pseudoinverse(V) %*% residuals) / (t(Y-a) %*% pseudoinverse(V) %*% (Y-a))
+		tmp$Rsq <- Rsq
+        return(tmp)
 	}
-	
+
 	#Informs the user that the summarization has begun, output model-dependent and dependent on whether the root theta is to be estimated
 	if(quiet==FALSE){
-		cat("Finished. Summarizing results.", "\n")	
+		cat("Finished. Summarizing results.", "\n")
 	}
-	
+
 	if(model == "TrendyM" | model == "TrendyMS"){
 		theta<-NULL
 		root.est <- out$solution[length(out$solution)]
 		theta$theta.est <- matrix(root.est, k+1, 2)
-		theta$theta.est[,2] <- 0 
+		theta$theta.est[,2] <- 0
 		theta$res <- expected.trendy(phy, edges, Rate.mat, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, root.value=root.est) - x
 	}else{
 		theta <- dev.theta(out$solution, index.mat, edges, mserr)
 	}
-	
+
 	#Calculates the Hessian for use in calculating standard errors and whether the maximum likelihood solution was found
 	if(diagn==TRUE){
 		h <- hessian(x=log(out$solution), func=dev, index.mat=index.mat, edges=edges, mserr=mserr, trendy=trendy)
@@ -514,10 +527,10 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 		}
 		if(simmap.tree==TRUE){
 			colnames(solution) <- colnames(solution.se) <- c(colnames(phy$mapped.edge))
-		}				
+		}
 		#Eigendecomposition of the Hessian to assess reliability of likelihood estimates
 		hess.eig<-eigen(h,symmetric=TRUE)
-		#If eigenvect is TRUE then the eigenvector and index matrix will appear in the list of objects 
+		#If eigenvect is TRUE then the eigenvector and index matrix will appear in the list of objects
 		eigval<-signif(hess.eig$values,2)
 		eigvect<-round(hess.eig$vectors, 2)
 		if(mserr=="est"){
@@ -526,9 +539,9 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 		}
 		else{
 			mserr.est<-NULL
-		}		
-		obj = list(loglik = loglik, AIC = -2*loglik+2*param.count,AICc=-2*loglik+(2*param.count*(ntips/(ntips-param.count-1))),model=model, param.count=param.count, solution=solution, mserr.est=mserr.est, theta=theta$theta.est, solution.se=solution.se, tot.states=tot.states, index.mat=index.mat, simmap.tree=simmap.tree, root.age=root.age, opts=opts, data=data, phy=phy, root.station=root.station, starting.vals=starting.vals, lb=lower, ub=upper, iterations=out$iterations, res=theta$res, eigval=eigval, eigvect=eigvect)
-		
+		}
+		obj = list(loglik = loglik, AIC = -2*loglik+2*param.count,AICc=-2*loglik+(2*param.count*(ntips/(ntips-param.count-1))),BIC=-2*loglik + log(ntips) * param.count, model=model, param.count=param.count, solution=solution, mserr.est=mserr.est, theta=theta$theta.est, solution.se=solution.se, tot.states=tot.states, index.mat=index.mat, simmap.tree=simmap.tree, root.age=root.age, opts=opts, data=data, phy=phy, root.station=root.station, starting.vals=starting.vals, lb=lower, ub=upper, iterations=out$iterations, res=theta$res, rsq=theta$Rsq, eigval=eigval, eigvect=eigvect)
+
 	}
 	if(diagn==FALSE){
 		solution<-matrix(out$solution[index.mat], dim(index.mat))
@@ -550,21 +563,21 @@ OUwie<-function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA",
 		else{
 			mserr.est<-NULL
 		}
-		obj = list(loglik = loglik, AIC = -2*loglik+2*param.count,AICc=-2*loglik+(2*param.count*(ntips/(ntips-param.count-1))),model=model, param.count=param.count, solution=solution, mserr.est=mserr.est, theta=theta$theta.est, tot.states=tot.states, index.mat=index.mat, simmap.tree=simmap.tree, root.age=root.age, opts=opts, data=data, phy=phy, root.station=root.station, starting.vals=starting.vals, lb=lower, ub=upper, iterations=out$iterations, res=theta$res)
+		obj = list(loglik = loglik, AIC = -2*loglik+2*param.count,AICc=-2*loglik+(2*param.count*(ntips/(ntips-param.count-1))),BIC=-2*loglik + log(ntips) * param.count, model=model, param.count=param.count, solution=solution, mserr.est=mserr.est, theta=theta$theta.est, tot.states=tot.states, index.mat=index.mat, simmap.tree=simmap.tree, root.age=root.age, opts=opts, data=data, phy=phy, root.station=root.station, starting.vals=starting.vals, lb=lower, ub=upper, iterations=out$iterations, res=theta$res, rsq=theta$Rsq)
 	}
-	class(obj)<-"OUwie"		
+	class(obj)<-"OUwie"
 	return(obj)
 }
 
 print.OUwie<-function(x, ...){
-	
+
 	ntips=Ntip(x$phy)
-	output<-data.frame(x$loglik,x$AIC,x$AICc,x$model,ntips, row.names="")
-	names(output)<-c("lnL","AIC","AICc","model","ntax")
+	output<-data.frame(x$loglik,x$AIC,x$AICc,x$BIC,x$model,ntips, row.names="")
+	names(output)<-c("lnL","AIC","AICc","BIC","model","ntax")
 	cat("\nFit\n")
 	print(output)
 	cat("\n")
-	
+
 	if (is.character(x$model)) {
 		if (x$model == "BM1" | x$model == "BMS" | x$model == "TrendyM" | x$model == "TrendyMS"){
 			param.est <- x$solution
@@ -605,6 +618,9 @@ print.OUwie<-function(x, ...){
 				cat("\nOptima\n")
 				print(theta.mat)
 				cat("\n")
+				cat("\nHalf life (another way of reporting alpha)\n")
+				print(log(2)/param.est['alpha',])
+				cat("\n")
 			}
 		}
 		if (x$root.station == TRUE){
@@ -624,10 +640,13 @@ print.OUwie<-function(x, ...){
 				cat("Optima\n")
 				print(theta.mat)
 				cat("\n")
+				cat("\nHalf life (another way of reporting alpha)\n")
+				print(log(2)/param.est['alpha',])
+				cat("\n")
 			}
 		}
 		if (x$root.station == FALSE){
-			if (x$model == "OUM"| x$model == "OUMV"| x$model == "OUMA" | x$model == "OUMVA"){ 
+			if (x$model == "OUM"| x$model == "OUMV"| x$model == "OUMA" | x$model == "OUMVA"){
 				param.est<- x$solution
 				theta.mat<-matrix(t(x$theta), 2, length(levels(x$tot.states))+1)
 				rownames(theta.mat)<-c("estimate", "se")
@@ -643,6 +662,9 @@ print.OUwie<-function(x, ...){
 				cat("Optima\n")
 				print(theta.mat)
 				cat("\n")
+				cat("\nHalf life (another way of reporting alpha)\n")
+				print(log(2)/param.est['alpha',])
+				cat("\n")
 			}
 		}
 	}
@@ -653,7 +675,7 @@ print.OUwie<-function(x, ...){
 		}
 		if(x$simmap.tree==TRUE){
 			colnames(index.matrix) <- colnames(x$phy$mapped.edge)
-		}				
+		}
 		#If any eigenvalue is less than 0 then the solution is not the maximum likelihood solution
 		if (any(x$eigval<0)) {
 			cat("The objective function may be at a saddle point -- check eigenvectors or try a simpler model", "\n")
@@ -661,7 +683,7 @@ print.OUwie<-function(x, ...){
 	}
 	else{
 		cat("Arrived at a reliable solution","\n")
-	}	
+	}
 }
 
 
@@ -680,7 +702,3 @@ MakeAgeTable <- function(phy, root.age=NULL){
     }
     return(table.ages)
 }
-
-
-
-
